@@ -19,6 +19,7 @@
 
 #include "question.h"
 
+#include <QCollator>
 #include <knewstuffcore_debug.h>
 #include <klocalizedstring.h>
 
@@ -36,7 +37,7 @@ namespace KNSCore
 {
 
 AtticaProvider::AtticaProvider(const QStringList &categories)
-    : mEntryJob(0)
+    : mEntryJob(nullptr)
     , mInitialized(false)
 {
     // init categories map with invalid categories
@@ -50,7 +51,7 @@ AtticaProvider::AtticaProvider(const QStringList &categories)
 }
 
 AtticaProvider::AtticaProvider(const Attica::Provider &provider, const QStringList &categories)
-    : mEntryJob(0)
+    : mEntryJob(nullptr)
     , mInitialized(false)
 {
     // init categories map with invalid categories
@@ -121,12 +122,30 @@ void AtticaProvider::listOfCategoriesLoaded(Attica::BaseJob *listJob)
     Attica::ListJob<Attica::Category> *job = static_cast<Attica::ListJob<Attica::Category>*>(listJob);
     Category::List categoryList = job->itemList();
 
+    QList<CategoryMetadata> categoryMetadataList;
     foreach (const Category &category, categoryList) {
         if (mCategoryMap.contains(category.name())) {
-            qCDebug(KNEWSTUFFCORE) << "Adding category: " << category.name();
-            mCategoryMap[category.name()] = category;
+            qCDebug(KNEWSTUFFCORE) << "Adding category: " << category.name() << category.displayName();
+            //If there is only the placeholder category, replace it
+            if (mCategoryMap.contains(category.name()) && !mCategoryMap.value(category.name()).isValid()) {
+                mCategoryMap.insert(category.name(), category);
+            } else {
+                mCategoryMap.insertMulti(category.name(), category);
+            }
+
+            CategoryMetadata categoryMetadata;
+            categoryMetadata.id = category.id();
+            categoryMetadata.name = category.name();
+            categoryMetadata.displayName = category.displayName();
+            categoryMetadataList << categoryMetadata;
         }
     }
+    std::sort(categoryMetadataList.begin(), categoryMetadataList.end(), [](const AtticaProvider::CategoryMetadata &i, const AtticaProvider::CategoryMetadata &j) -> bool {
+        const QString a(i.displayName.isEmpty() ? i.name : i.displayName);
+        const QString b(j.displayName.isEmpty() ? j.name : j.displayName);
+
+        return (QCollator().compare(a, b) < 0);
+    });
 
     bool correct = false;
     for(auto it = mCategoryMap.cbegin(), itEnd = mCategoryMap.cend(); it!=itEnd; ++it) {
@@ -140,6 +159,7 @@ void AtticaProvider::listOfCategoriesLoaded(Attica::BaseJob *listJob)
     if (correct) {
         mInitialized = true;
         emit providerInitialized(this);
+        emit categoriesMetadataLoded(categoryMetadataList);
     } else {
         emit signalError(i18n("All categories are missing"));
     }
@@ -154,7 +174,7 @@ void AtticaProvider::loadEntries(const KNSCore::Provider::SearchRequest &request
 {
     if (mEntryJob) {
         mEntryJob->abort();
-        mEntryJob = 0;
+        mEntryJob = nullptr;
     }
 
     mCurrentRequest = request;
@@ -188,7 +208,7 @@ void AtticaProvider::loadEntries(const KNSCore::Provider::SearchRequest &request
     } else {
         categoriesToSearch.reserve(request.categories.size());
         foreach (const QString &categoryName, request.categories) {
-            categoriesToSearch.append(mCategoryMap.value(categoryName));
+            categoriesToSearch.append(mCategoryMap.values(categoryName));
         }
     }
 
@@ -256,7 +276,7 @@ void AtticaProvider::categoryContentsLoaded(BaseJob *job)
 
     qCDebug(KNEWSTUFFCORE) << "loaded: " << mCurrentRequest.hashForRequest() << " count: " << entries.size();
     emit loadingFinished(mCurrentRequest, entries);
-    mEntryJob = 0;
+    mEntryJob = nullptr;
 }
 
 Attica::Provider::SortMode AtticaProvider::atticaSortMode(const SortMode &sortMode)
@@ -319,7 +339,6 @@ void AtticaProvider::accountBalanceLoaded(Attica::BaseJob *baseJob)
         if(question.ask() == Question::YesResponse) {
             ItemJob<DownloadItem> *job = m_provider.downloadLink(entry.uniqueId(), QString::number(pair.second));
             connect(job, &BaseJob::finished, this, &AtticaProvider::downloadItemLoaded);
-            connect(job, SIGNAL(jobStarted(QNetworkReply*)), SLOT(atticaJobStarted(QNetworkReply*)));
             mDownloadLinkJobs[job] = qMakePair(entry, pair.second);
             job->start();
         } else {
@@ -362,7 +381,6 @@ void AtticaProvider::vote(const EntryInternal &entry, uint rating)
 {
     PostJob *job = m_provider.voteForContent(entry.uniqueId(), rating);
     connect(job, &BaseJob::finished, this, &AtticaProvider::votingFinished);
-    connect(job, SIGNAL(jobStarted(QNetworkReply*)), SLOT(atticaJobStarted(QNetworkReply*)));
     job->start();
 }
 
@@ -378,7 +396,6 @@ void AtticaProvider::becomeFan(const EntryInternal &entry)
 {
     PostJob *job = m_provider.becomeFan(entry.uniqueId());
     connect(job, &BaseJob::finished, this, &AtticaProvider::becomeFanFinished);
-    connect(job, SIGNAL(jobStarted(QNetworkReply*)), SLOT(atticaJobStarted(QNetworkReply*)));
     job->start();
 }
 
